import json

from django import template
from django.core.serializers.json import DjangoJSONEncoder
from django.utils.safestring import mark_safe

from .. import app_settings

register = template.Library()


@register.filter(is_safe=True)
def js(obj):
    """ Transform a python object so it can be safely used in javascript/JSON. """
    return mark_safe(json.dumps(obj, cls=DjangoJSONEncoder))


@register.inclusion_tag('pwa.html', takes_context=True)
def progressive_web_app_meta(context):
    # Pass all PWA_* settings into the template
    return {
        setting_name: getattr(app_settings, setting_name)
        for setting_name in dir(app_settings)
        if setting_name.startswith('PWA_')
    }



import re
from django.db.models import Value
from django.utils.safestring import mark_safe
from django.conf import settings

class CustomValueDb(Value):
    def __init__(self, value, output_field=None):
        from django.db.models import IntegerField, CharField, FloatField, DecimalField, BooleanField, DateField, DateTimeField
        type_value = type(value).__name__.lower()
        if not output_field:
            if "int" in type_value:
                output_field = IntegerField()
            if "str" in type_value:
                output_field = CharField()
            if "float" in type_value:
                output_field = FloatField()
            if "decimal" in type_value:
                output_field = DecimalField()
            if "bool" in type_value:
                output_field = BooleanField()
            if "date" in type_value:
                output_field = DateField()
            if "datetime" in type_value:
                output_field = DateTimeField()
        super().__init__(value, output_field=output_field)


class FormException(Exception):
    def __init__(self, form, prefix="", sufix=""):
        super().__init__("Error en el formulario")
        alerta = ""
        if isinstance(form, list) or isinstance(form, tuple):
            self.errors = []
            for x in form:
                for k, v in x.errors.items():
                    self.errors.append({prefix+k+sufix: v[0]})
        else:
            self.errors = [{prefix+k+sufix: v[0]} for k, v in form.errors.items()]
        for x in form.errors.get('__all__') or []:
            for key in getattr(settings, 'CONSTRAINT_MSG', {}).keys():
                if re.search(f"\\b{key}\\b", x):
                    alerta += f'<div>{getattr(settings, "CONSTRAINT_MSG", {})[key]}</div>'
        self.dict_error = {
            'error': True,
            "form": self.errors,
            "message": "Datos incorrectos, revise la información registrada.",
            "alerta": mark_safe(alerta)
        }


import re
from django.contrib import messages
from django.db import transaction, IntegrityError
from django.http import JsonResponse
from django.shortcuts import redirect
from django.conf import settings
from for_django_projects.utils.funciones import redirectAfterPostGet


def custom_atomic_request(func):
    from for_django_projects.utils.custom_models import FormException
    import sys
    def validate_request(*args, **kwargs):
        res_json = []
        request = args[0]
        has_except = False
        error_message = ""
        if request.method == "POST":
            action = request.POST.get("action")
            try:
                with transaction.atomic():
                    val_func = func(*args, **kwargs)
            except ValueError as ex:
                res_json.append({'error': True,
                                 "message": str(ex)
                                 })
                val_func = JsonResponse(res_json, safe=False)
                has_except = True
                error_message = str(ex)
            except FormException as ex:
                res_json.append(ex.dict_error)
                val_func = JsonResponse(res_json, safe=False)
                has_except = True
                error_message = "Formulario no válido"
            except IntegrityError as ex:
                has_except = True
                msg = str(ex)
                error_message = "Integrity Error"
                for key in getattr(settings, 'CONSTRAINT_MSG', {}).keys():
                    if re.search(f"\\b{key}\\b", msg):
                        error_message = getattr(settings, 'CONSTRAINT_MSG', {}).get(key) or 'Integrity Error'
                if request.user.is_superuser:
                    error_message = f"{error_message} | {msg}"
                res_json.append(
                    {
                        'error': has_except,
                        "message": error_message
                    }
                )
                val_func = JsonResponse(res_json, safe=False)
            except Exception as ex:
                res_json.append({'error': True,
                                 "message": "Intente Nuevamente"
                                 })
                val_func = JsonResponse(res_json, safe=False)
                has_except = True
                error_message = "Intente Nuevamente"
                if request.user.is_superuser:
                    error_message = f"{error_message} | {ex}"
        elif request.method == "GET":
            val_func = func(*args, **kwargs)
        if has_except and not request.is_ajax:
            messages.error(request, error_message)
            val_func = redirect(redirectAfterPostGet(request))
        return val_func

    return validate_request


def sync_to_async_function(f):
    import threading
    def threading_func(*a, **kw):
        t = threading.Thread(target=f, args=a, kwargs=kw)
        t.start()
        # t.join()

    return threading_func


def validate_atomic_request(func):
    from for_django_projects.utils.custom_models import FormException
    import sys
    def validate_request(*args, **kwargs):
        res_json = {}
        request = args[0]
        has_except = False
        error_message = ""
        if request.method == "POST":
            action = request.POST.get("action")
            try:
                with transaction.atomic():
                    val_func = func(*args, **kwargs)
            except ValueError as ex:
                res_json = {"message": str(ex)}
                val_func = JsonResponse(res_json, status=202)
                has_except = True
                error_message = str(ex)
            except FormException as ex:
                res_json = ex.dict_error
                val_func = JsonResponse(res_json, status=202)
                has_except = True
                error_message = "Formulario no válido"
            except IntegrityError as ex:
                has_except = True
                msg = str(ex)
                error_message = "Integrity Error"
                for key in getattr(settings, 'CONSTRAINT_MSG', {}).keys():
                    if re.search(f"\\b{key}\\b", msg):
                        error_message = getattr(settings, 'CONSTRAINT_MSG', {}).get(key) or 'Integrity Error'
                if request.user.is_superuser:
                    error_message = f"{error_message} | {msg}"
                res_json = {
                    "message": error_message
                }
                val_func = JsonResponse(res_json, status=202)
            except Exception as ex:
                error_message = "Intente Nuevamente"
                if request.user.is_superuser:
                    error_message = f"{error_message} | {ex}"
                res_json = {
                    "message": error_message
                }
                val_func = JsonResponse(res_json, status=202)
                has_except = True
        elif request.method == "GET":
            val_func = func(*args, **kwargs)
        if has_except and not request.is_ajax:
            messages.error(request, error_message)
            val_func = redirect(redirectAfterPostGet(request))
        return val_func

    return validate_request


import json
from django.core import signing


def redirectAfterPostGet(request, campos_add={}):
    dict_url_vars = request.GET.get('dict_url_vars') or request.POST.get('dict_url_vars') or ""
    if dict_url_vars:
        try:
            dict_url_vars = json.loads(get_decrypt(dict_url_vars)[1]).get(request.path) or ""
        except Exception as ex:
            print(ex)
    salida = "?action=add&" if '_add' in request.POST else request.path + "?"
    if '_add' in request.POST:
        for k, v in campos_add.items():
            salida += "&{}={}".format(k, v)
    return salida + "{}".format(dict_url_vars)


def ip_client_address(request):
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        ip = x_forwarded_for.split(',')[-1].strip()
    else:
        ip = request.META.get('REMOTE_ADDR')
    return ip


def get_encrypt(values):
    try:
        return True, signing.dumps(values, compress=True)
    except Exception as ex:
        return False, str(ex)


def get_decrypt(cyphertxt):
    try:
        return True, signing.loads(cyphertxt)
    except Exception as ex:
        return False, str(ex)


def remover_espacios_de_mas(valor: str) -> str:
    import re
    return re.sub("\s+", " ", (valor or "").strip())


def formarCondicion(c, busqueda):
    from django.db.models import Q
    if c.endswith("__islisttype"):
        return Q(**{c.replace("__islisttype", ""): [busqueda]})
    else:
        return Q(**{c: busqueda})
def campoSinFiltro(c):
    filtros = ["__icontains", "__contains", "__startswith", "__istartswith", "__endswith", "__iendswith", "__range", "__search"]
    for f in filtros:
        if c.endswith(f):
            return False
    return True
def criterioBusquedaDinamico(criterio: str, campos: list, isPostgres=True):
    '''Si el modelo no proviene de una base de datos postgres set isPostgres=False'''
    from django.db.models import Q;from django.utils.text import smart_split
    filtros = Q()
    criterio_list = smart_split(remover_espacios_de_mas(criterio))
    for cl in criterio_list:
        cri = cl.strip()
        f = Q()
        if cri:
            for c in campos:
                if campoSinFiltro(c):
                    if cri.startswith("%") and cri.endswith("%"):
                        c = c + "__icontains"
                    elif cri.startswith("%"):
                        c = c + "__iendswith"
                    elif cri.endswith("%"):
                        c = c + "__istartswith"
                    else:
                        if isPostgres:
                            f |= formarCondicion(c + "__search", cri.replace("%", ""))
                        c = c + "__icontains"
                f |= formarCondicion(c, cri.replace("%", ""))
            filtros &= (f)
    return (filtros)